from six import StringIO
from insights import dr, make_response, rule
from insights.formats.text import HumanReadableFormat
from insights.formats._json import JsonFormat


@rule()
def report():
    return make_response("ERROR", foo="bar")


def test_human_readable():
    broker = dr.Broker()
    output = StringIO()
    with HumanReadableFormat(broker, stream=output):
        dr.run(report, broker=broker)
    output.seek(0)
    data = output.read()
    assert "foo" in data
    assert "bar" in data


def test_json_format():
    broker = dr.Broker()
    output = StringIO()
    with JsonFormat(broker, stream=output):
        dr.run(report, broker=broker)
    output.seek(0)
    data = output.read()
    assert "foo" in data
    assert "bar" in data



"""
grubby - command ``/usr/sbin/grubby``
=====================================

This parser returns the output of the ``grubby --default-index`` command.

Examples:

    >>> grubby_default_index.default_index
    0
"""

from insights.specs import Specs
from .. import parser, Parser
from . import SkipException


@parser(Specs.grubby_default_index)
class GrubbyDefaultIndex(Parser):
    """
    This parser will parse the output of command ``grubby --default-index``.

    Attributes:
        default_index (int): the numeric index of the current default boot entry, count from 0
    """
    def parse_content(self, content):
        if content and content[0].isdigit():
            self.default_index = int(content[0])
        else:
            raise SkipException('Invalid default index value.')



from __future__ import print_function
import six
import sys
from insights import dr, rule


_FORMATTERS = {}


def get_formatter(name):
    """
    Looks up a formatter class given a prefix to it.
    The names are sorted, and the first matching class is returned.
    """
    for k in sorted(_FORMATTERS):
        if k.startswith(name):
            return _FORMATTERS[k]


class FormatterAdapterMeta(type):
    """ Automatically registers subclasses for later lookup. """

    def __init__(cls, name, bases, dct):
        if name not in ("FormatterAdapter", "EvaluatorFormatterAdapter"):
            _FORMATTERS[dr.get_name(cls)] = cls
        super(FormatterAdapterMeta, cls).__init__(name, bases, dct)


class FormatterAdapter(six.with_metaclass(FormatterAdapterMeta)):

    @staticmethod
    def configure(p):
        """ Override to add arguments to the ArgumentParser. """
        pass

    def __init__(self, args=None):
        """ Subclasses get passed the parsed args. """
        pass

    def preprocess(self, broker):
        """
        Called before any components have been run. Useful for registering
        observers.
        """
        pass

    def postprocess(self, broker):
        """
        Called after all components have been run. Useful for interrogating
        the broker for final state.
        """


class Formatter(object):
    def __init__(self, broker, stream=sys.stdout):
        self.broker = broker
        self.stream = stream

    def __enter__(self):
        self.preprocess()
        return self

    def __exit__(self, _type, value, tb):
        self.postprocess()

    def preprocess(self):
        pass

    def postprocess(self):
        pass


class EvaluatorFormatter(Formatter):
    def preprocess(self):
        from insights.core.evaluators import SingleEvaluator
        self.evaluator = SingleEvaluator(broker=self.broker)

    def postprocess(self):
        self.evaluator.post_process()
        print(self.dump(self.evaluator.get_response()), file=self.stream)

    def dump(self, data):
        raise NotImplemented("Subclasses must implement the dump method.")


class EvaluatorFormatterAdapter(FormatterAdapter):
    """
    Base class for formatters that want to serialize a SingleEvaluator after
    execution.
    """
    Impl = None

    def __init__(self, args=None):
        if args:
            hn = "insights.combiners.hostname"
            args.plugins = ",".join([args.plugins, hn]) if args.plugins else hn

    def preprocess(self, broker):
        self.formatter = self.Impl(broker)
        self.formatter.preprocess()

    def postprocess(self, broker):
        self.formatter.postprocess()


RENDERERS = {}

try:
    from jinja2 import Template

    def get_content(obj, key):
        mod = sys.modules[obj.__module__]
        c = getattr(mod, "CONTENT", None)
        if c:
            if isinstance(c, dict):
                if key:
                    return c.get(key)
            else:
                return c

    def format_rule(comp, val):
        content = get_content(comp, val.get_key())
        if content:
            return Template(content).render(val)
        return str(val)

    RENDERERS[rule] = format_rule
except:
    pass


def render(comp, val):
    _type = dr.get_component_type(comp)
    func = RENDERERS.get(_type)
    return func(comp, val) if func else str(val)



from ...parsers import lssap, ParseException
from ...tests import context_wrap
from ...util import keys_in
import pytest

Lssap_BAD = """
 - lssap version 1.0 -
 ==========================================
   SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
   HA2|  16|       D16|         lu0417
==========================================
""".strip()

Lssap_BAD1 = """
   HA2|  16|       D16|         lu0417
""".strip()

Lssap_BAD2 = ""

Lssap_nw_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       D16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
  HA2|  22|       D22|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
  HA2|  50|       D50|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D50/exe
  HA2|  51|       D51|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D51/exe
  HA2|  52|       D52|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D52/exe
==========================================
""".strip()

Lssap_hana_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       HDB16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/HDB16/exe
==========================================
""".strip()

Lssap_ascs_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       ASCS16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/ASCS16/exe
==========================================
""".strip()

Lssap_bad_instance_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       B16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
  HA2|  22|       QR22|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
  HA2|  50|       DS0|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D50/exe
==========================================
""".strip()

Lssap_runAll_Test = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       D16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
  HA2|  22|       D22|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
  HA2|  16|       HDB16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/HDB16/exe
  HA2|  16|       ASCS16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/ASCS16/exe
==========================================
""".strip()

Lssap_BAD = """
/usr/sap/hostctrl/lssap: command or file not found
"""

Lssap_BAD_2 = """
HA2 | N16|  D16
""".strip()

Lssap_BAD_3 = """
HA2 | N16 | D16
HB2 | foo | bar
""".strip()


def test_lists():
    sap = lssap.Lssap(context_wrap(Lssap_runAll_Test))
    assert sap.instances == ["D16", "D22", "HDB16", "ASCS16"]
    assert sap.sid == ["HA2"]
    assert sap.version("D16") == "749, patch 10, changelist 1698137"


def test_lssap_netweaver():
    sap = lssap.Lssap(context_wrap(Lssap_nw_TEST))
    data = sap.data
    assert keys_in(["SID", "Nr", "Instance", "SAPLOCALHOST", "Version", "DIR_EXECUTABLE"], data[0])
    assert data[0] == {"SID": "HA2", "Nr": "16", "Instance": "D16", "SAPLOCALHOST": "lu0417",
                       "Version": "749, patch 10, changelist 1698137",
                       "DIR_EXECUTABLE": "/usr/sap/HA2/D16/exe"}
    assert data[2]["Instance"] == "D50"
    assert data[-1]["SAPLOCALHOST"] == "lu0417"
    assert sap.is_netweaver() is True
    assert sap.is_hana() is False
    assert sap.is_ascs() is False


def test_lssap_hana():
    sap = lssap.Lssap(context_wrap(Lssap_hana_TEST))
    data = sap.data
    assert data[0] == {"SID": "HA2", "Nr": "16", "Instance": "HDB16", "SAPLOCALHOST": "lu0417",
                       "Version": "749, patch 10, changelist 1698137",
                       "DIR_EXECUTABLE": "/usr/sap/HA2/HDB16/exe"}
    assert data[0]["SAPLOCALHOST"] == "lu0417"
    assert sap.is_hana() is True
    assert sap.is_netweaver() is False
    assert sap.is_ascs() is False


def test_lssap_ascs():
    sap = lssap.Lssap(context_wrap(Lssap_ascs_TEST))
    assert sap.is_hana() is False
    assert sap.is_netweaver() is False
    assert sap.is_ascs() is True


def test_all():
    sap = lssap.Lssap(context_wrap(Lssap_runAll_Test))
    data = sap.data
    assert data[1] == {"SID": "HA2", "Nr": "22", "Instance": "D22", "SAPLOCALHOST": "lu0417",
                       "Version": "749, patch 10, changelist 1698137",
                       "DIR_EXECUTABLE": "/usr/sap/HA2/D22/exe"}
    assert sap.is_netweaver() is True
    assert sap.is_hana() is True
    assert sap.is_ascs() is True


def test_fail():
    with pytest.raises(ParseException) as excinfo:
        lssap.Lssap(context_wrap(Lssap_BAD))
    assert "Lssap: Unable to parse 1 line(s) of content: (['/usr/sap/hostctrl/lssap: command or file not found'])" in str(excinfo)

    with pytest.raises(ParseException) as excinfo:
        lssap.Lssap(context_wrap('test'))
    assert "Lssap: Unable to parse 1 line(s) of content: (['test'])" in str(excinfo)

    with pytest.raises(ParseException) as excinfo:
        lssap.Lssap(context_wrap(Lssap_BAD_2))
    assert "Lssap: Unable to parse 1 line(s) of content: (['HA2 | N16|  D16'])" in str(excinfo)

    with pytest.raises(ParseException) as excinfo:
        lssap.Lssap(context_wrap(Lssap_BAD_3))
    assert "Lssap: Unable to parse 2 line(s) of content: (['HA2 | N16 | D16', 'HB2 | foo | bar'])" in str(excinfo)

    with pytest.raises(ParseException) as excinfo:
        lssap.Lssap(context_wrap(Lssap_bad_instance_TEST))
    assert "Lssap: Invalid instance parsed in content: ['B', 'QR', 'DS']" in str(excinfo)



"""
Red Hat Release
===============

Combiner for Red Hat Release information. It uses the results of
the ``Uname`` parser and the ``rht_release`` parser to determine the release
major and minor version.  ``Uname`` is the preferred source of data. The Red
Hat Release is in obtained from the system in the form major.minor.  For
example for a Red Hat Enterprise Linux 7.2 system, the release would be
major = 7 and minor = 2.  The returned values are in integer format.Re

Examples:
    >>> rh_release = shared[redhat_release]
    >>> rh_release.major
    7
    >>> rh_release.minor
    2
    >>> rh_release
    Release(major=7, minor=2)

"""

from collections import namedtuple
from insights.core.plugins import combiner
from insights.parsers.redhat_release import RedhatRelease as rht_release
from insights.parsers.uname import Uname
from insights.core.serde import deserializer, serializer

Release = namedtuple("Release", field_names=["major", "minor"])
"""namedtuple: Type for storing the release information."""


@serializer(Release)
def serialize(obj):
    return {"major": obj.major, "minor": obj.minor}


@deserializer(Release)
def deserialize(_type, obj):
    return Release(**obj)


@combiner([rht_release, Uname])
def redhat_release(rh_release, un):
    """Check uname and redhat-release for rhel major/minor version.

    Prefer uname to redhat-release.

    Returns:
        Release: A named tuple with `major` and `minor` version
        components.

    Raises:
        Exeption: If the version can't be determined even though a Uname
            or RedhatRelease was provided.
    """

    if un and un.release_tuple[0] != -1:
        return Release(*un.release_tuple)

    if rh_release:
        return Release(rh_release.major, rh_release.minor)

    raise Exception("Unabled to determine release.")



from insights.parsers.systemd import config
from insights.tests import context_wrap


SYSTEMD_DOCKER = """
[Unit]
Description=Docker Application Container Engine
Documentation=http://docs.docker.com
After=network.target
Wants=docker-storage-setup.service
Requires=rhel-push-plugin.socket

[Service]
Type=notify
NotifyAccess=all
EnvironmentFile=-/etc/sysconfig/docker
EnvironmentFile=-/etc/sysconfig/docker-storage
EnvironmentFile=-/etc/sysconfig/docker-network
Environment=GOTRACEBACK=crash
ExecStart=/bin/sh -c '/usr/bin/docker-current daemon \\
          --authorization-plugin=rhel-push-plugin \\
          --exec-opt native.cgroupdriver=systemd \\
          $OPTIONS \\
          $DOCKER_STORAGE_OPTIONS \\
          $DOCKER_NETWORK_OPTIONS \\
          $ADD_REGISTRY \\
          $BLOCK_REGISTRY \\
          $INSECURE_REGISTRY \\
          2>&1 | /usr/bin/forward-journald -tag docker'
LimitNOFILE=1048576
LimitNPROC=1048576
LimitCORE=infinity
TimeoutStartSec=0
MountFlags=slave
Restart=on-abnormal
StandardOutput=null
StandardError=null

[Install]
WantedBy=multi-user.target
""".strip()


SYSTEMD_OPENSHIFT_NODE = """
[Unit]
Description=Atomic OpenShift Node
After=docker.service
After=openvswitch.service
Wants=docker.service
Documentation=https://github.com/openshift/origin

[Service]
Type=notify
EnvironmentFile=/etc/sysconfig/atomic-openshift-node
Environment=GOTRACEBACK=crash
ExecStart=/usr/bin/openshift start node --config=${CONFIG_FILE} $OPTIONS
LimitNOFILE=65536
LimitCORE=infinity
WorkingDirectory=/var/lib/origin/
SyslogIdentifier=atomic-openshift-node
Restart=always
RestartSec=5s
OOMScoreAdjust=-999
ExecStartPost=/usr/bin/sleep 10
ExecStartPost=/usr/sbin/sysctl --system

[Install]
WantedBy=multi-user.target

""".strip()


SYSTEMD_SYSTEM_CONF = """
#  This file is part of systemd.
#
#  systemd is free software; you can redistribute it and/or modify it
#  under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 2.1 of the License, or
#  (at your option) any later version.
#
# Entries in this file show the compile time defaults.
# You can change settings by editing this file.
# Defaults can be restored by simply deleting this file.
#
# See systemd-system.conf(5) for details.

[Manager]
#LogLevel=info
#LogTarget=journal-or-kmsg
#LogColor=yes
#LogLocation=no
#DumpCore=yes
#CrashShell=no
#ShowStatus=yes
#CrashChVT=1
#CPUAffinity=1 2
#JoinControllers=cpu,cpuacct net_cls,net_prio
RuntimeWatchdogSec=0
ShutdownWatchdogSec=10min
#CapabilityBoundingSet=
#SystemCallArchitectures=
#TimerSlackNSec=
#DefaultTimerAccuracySec=1min
#DefaultStandardOutput=journal
#DefaultStandardError=inherit
#DefaultTimeoutStartSec=90s
#DefaultTimeoutStopSec=90s
#DefaultRestartSec=100ms
#DefaultStartLimitInterval=10s
#DefaultStartLimitBurst=5
#DefaultEnvironment=
#DefaultCPUAccounting=no
#DefaultBlockIOAccounting=no
#DefaultMemoryAccounting=no
#DefaultLimitCPU=
#DefaultLimitFSIZE=
#DefaultLimitDATA=
#DefaultLimitSTACK=
#DefaultLimitCORE=
#DefaultLimitRSS=
#DefaultLimitNOFILE=
#DefaultLimitAS=
#DefaultLimitNPROC=
#DefaultLimitMEMLOCK=
#DefaultLimitLOCKS=
#DefaultLimitSIGPENDING=
#DefaultLimitMSGQUEUE=
#DefaultLimitNICE=
#DefaultLimitRTPRIO=
#DefaultLimitRTTIME=
""".strip()


def test_docker():
    docker_service = config.SystemdDocker(context_wrap(SYSTEMD_DOCKER))
    assert docker_service.data["Unit"]["After"] == "network.target"
    assert docker_service.data["Service"]["NotifyAccess"] == "all"
    assert docker_service.data["Service"]["Environment"] == "GOTRACEBACK=crash"
    assert docker_service.data["Install"]["WantedBy"] == "multi-user.target"
    assert list(docker_service.data["Install"].keys()) == ["WantedBy"]
    assert docker_service.data["Service"]["ExecStart"] == "/bin/sh -c '/usr/bin/docker-current daemon --authorization-plugin=rhel-push-plugin --exec-opt native.cgroupdriver=systemd $OPTIONS $DOCKER_STORAGE_OPTIONS $DOCKER_NETWORK_OPTIONS $ADD_REGISTRY $BLOCK_REGISTRY $INSECURE_REGISTRY 2>&1 | /usr/bin/forward-journald -tag docker'"


def test_openshift_node():
    openshift_node_service = config.SystemdOpenshiftNode(context_wrap(SYSTEMD_OPENSHIFT_NODE))
    assert openshift_node_service.data["Unit"]["Wants"] == "docker.service"
    assert openshift_node_service.data["Unit"]["After"] == ['docker.service', 'openvswitch.service']


def test_systemd_common_conf():
    common_conf_info = config.SystemdSystemConf(context_wrap(SYSTEMD_SYSTEM_CONF))
    assert "Manager" in common_conf_info
    assert common_conf_info["Manager"]["RuntimeWatchdogSec"] == "0"
    assert common_conf_info["Manager"]["ShutdownWatchdogSec"] == "10min"



#!/usr/bin/env python
"""
Create a mapping of specs, rules and filters for each parser

To run this script execute the following commands:
$ cd insights-core
$ source bin/activate
$ cd ../insights-plugins
$ export PYTHONPATH=./
$ ../insights-core/insights/tools/specs_to_rules.py > report.html

"""
import datetime
import logging
from jinja2 import Environment
from operator import itemgetter

from insights.core import dr, filters
from insights.core import spec_factory as sf
from insights.core.plugins import is_rule
from insights.specs.default import DefaultSpecs
from insights.specs import Specs


logging.basicConfig(level=logging.INFO)


REPORT = """
<html><head><title>Spec to Rule Mapping Report ({{ report_date }})</title>
<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/4.0.0-beta/css/bootstrap.min.css" integrity="sha384-/Y6pD6FV/Vv2HJnA6t+vslU6fwYXjCFtcEpHbNJ0lyAFsXTsjBbfaDjzALeQsN6M" crossorigin="anonymous">
</head>
<body>
<h2>Spec to Rule Mapping Report {{ report_date }}</h2>
<p>This is the mapping of Red Hat Insights Specs to Insights Rules they affect.</p>

<table class="table table-striped table-bordered">
<thead>
<tr><th>Spec Name</th><th>Spec</th><th>Rules</th><th>Filters</th></tr>
</thead>
<tbody>
{% for info in infos %}
    <tr>
        <td>{{ info.name }}</td>
        <td><code>{{ info.description }}</code></td>
        <td>{{ info.rules }}</td>
        <td>{{ info.filters }}</td>
    </tr>
{% endfor %}
</tbody>
</table>
</body></html>
""".strip()


def get_all(root, method=dr.get_dependencies):
    for d in method(root):
        yield d
        for c in get_all(d, method=method):
            yield c


def load_plugins():
    """ Load the plugins we care about """
    dr.load_components("insights.specs.default")
    dr.load_components("insights.parsers")
    dr.load_components("insights.combiners")
    dr.load_components("telemetry.rules.plugins")
    dr.load_components("prodsec")


def get_specs():
    """
    Get default datasources that implement a `RegistryPoint` defined in
    `insights.specs.Specs`
    """
    vals = vars(Specs).values()
    sd = sf.SpecDescriptor
    names = [d.__get__(d, None).__name__ for d in vals if isinstance(d, sd)]
    return [d for d in [getattr(DefaultSpecs, name, None) for name in names] if d]


def li(item):
    return "<li>%s</li>" % item


def ul(lst):
    return "<ul>\n%s\n</ul>\n" % "\n".join(li(i) for i in lst)


def get_description(spec):
    if isinstance(spec, sf.simple_file):
        return "Path: %s" % spec.path
    if isinstance(spec, sf.glob_file):
        return "All files matching any of%sIgnore: %s" % (ul(spec.patterns), spec.ignore)
    if isinstance(spec, sf.simple_command):
        return "Command: %s" % spec.cmd
    if isinstance(spec, sf.listdir):
        return "Directory Listing: %s" % spec.path
    if isinstance(spec, sf.foreach_execute):
        return "For each element in%s Command: %s" % (ul([get_description(spec.provider)]), spec.cmd)
    if isinstance(spec, sf.foreach_collect):
        return "For each element in%s Path: %s" % (ul([get_description(spec.provider)]), spec.path)
    if isinstance(spec, sf.first_of):
        return "First of%s" % ul(get_description(d) for d in spec.deps)
    if isinstance(spec, sf.first_file):
        return "First file of%s" % ul(spec.files)
    return (spec.__doc__ or "").replace("\n", "<br />")


def get_rules(spec):
    return [r for r in get_all(spec, method=dr.get_dependents) if is_rule(r)]


def get_infos(specs):
    def get_info(spec):
        return {
            "name": dr.get_simple_name(spec),
            "description": get_description(spec),
            "rules": ul(sorted(dr.get_name(r) for r in get_rules(spec))),
            "filters": ul(sorted(filters.get_filters(spec))),
        }
    return sorted((get_info(s) for s in specs), key=itemgetter("name"))


def main():
    today = datetime.date.today().strftime("%B %d, %Y")
    load_plugins()
    specs = get_specs()
    infos = get_infos(specs)

    template = Environment().from_string(REPORT)
    report = template.render(report_date=today, infos=infos)
    print(report)


if __name__ == "__main__":
    main()



"""
The plugins module defines the components used by the rest of Insights and
specializes their interfaces and execution model where required.
"""

import logging
import traceback

from insights.core import dr
from insights.util.subproc import CalledProcessError
from insights import settings

log = logging.getLogger(__name__)


class ContentException(dr.SkipComponent):
    """ Raised whenever a datasource fails to get data. """
    pass


class datasource(dr.ComponentType):
    """ Decorates a component that one or more Parsers will consume. """
    multi_output = False
    raw = False
    filterable = False

    def invoke(self, broker):
        try:
            return self.component(broker)
        except ContentException as ce:
            log.debug(ce)
            broker.add_exception(self.component, ce, traceback.format_exc())
            raise dr.SkipComponent()
        except CalledProcessError as cpe:
            log.debug(cpe)
            broker.add_exception(self.component, cpe, traceback.format_exc())
            raise dr.SkipComponent()


class parser(dr.ComponentType):
    """
    Decorates a component responsible for parsing the output of a
    datasource.
    """
    def __init__(self, dep, group=dr.GROUPS.single):
        super(parser, self).__init__(dep, group=group)

    def invoke(self, broker):
        dep_value = broker[self.requires[0]]
        if not isinstance(dep_value, list):
            return self.component(dep_value)

        results = []
        for d in dep_value:
            try:
                r = self.component(d)
                if r is not None:
                    results.append(r)
            except dr.SkipComponent:
                pass
            except Exception as ex:
                tb = traceback.format_exc()
                log.warn(tb)
                broker.add_exception(self.component, ex, tb)

        if not results:
            log.debug("All failed: %s" % dr.get_name(self.component))
            raise dr.SkipComponent()
        return results


class metadata(parser):
    """ Used for old cluster uber-archives. """
    # TODO: Mark deprecated
    requires = ["metadata.json"]


class combiner(dr.ComponentType):
    """ ComponentType for a component that composes other components. """
    pass


class rule(dr.ComponentType):
    """
    ComponentType for a component that can see all parsers and combiners for a
    single host.
    """
    def process(self, broker):
        """
        Ensures dependencies have been met before delegating to `self.invoke`.
        """
        if any(i in broker for i in dr.IGNORE.get(self.component, [])):
            raise dr.SkipComponent()
        missing = self.get_missing_dependencies(broker)
        if missing:
            details = dr.stringify_requirements(missing)
            return _make_skip(dr.get_name(self.component),
                    reason="MISSING_REQUIREMENTS", details=details)
        r = self.invoke(broker)
        if r is None:
            raise dr.SkipComponent()
        if not isinstance(r, Response):
            raise Exception("rules must return Response objects.")
        return r


class condition(dr.ComponentType):
    """
    ComponentType for a component used by rules that allows automated
    statistical analysis.
    """
    pass


class incident(dr.ComponentType):
    """
    ComponentType for a component used by rules that allows automated
    statistical analysis.
    """
    pass


class fact(dr.ComponentType):
    """
    ComponentType for a component that surfaces a dictionary or list of
    dictionaries that will be used later by cluster rules.
    """
    pass


def is_type(component, _type):
    try:
        return issubclass(dr.get_component_type(component), _type)
    except:
        return False


def is_datasource(component):
    return is_type(component, datasource)


def is_parser(component):
    return is_type(component, parser)


def is_combiner(component):
    return is_type(component, combiner)


def is_rule(component):
    return is_type(component, rule)


def is_component(obj):
    return bool(dr.get_component_type(obj))


class ValidationException(Exception):
    def __init__(self, msg, r=None):
        if r:
            msg = "%s: %s" % (msg, r)
        super(ValidationException, self).__init__(msg)


class Response(dict):
    """
    Response is the base class of response types that can be returned from
    rules.

    Subclasses must call __init__ of this class via super() and must provide
    the response_type class attribute.

    The key_name class attribute is optional, but if one is specified, the
    first argument to __init__ must not be None. If key_name is None, then
    the first argument to __init__ should be None. It's best to override
    __init__ in subclasses so users aren't required to pass None explicitly.
    """

    response_type = None
    """
    response_type is something like 'rule', 'metadata', 'fingerprint', etc. It
    is how downstream systems identify the type of information returned by a
    rule.
    """

    key_name = None
    """
    key_name is something like 'error_key', 'fingerprint_key', etc.  It is the
    key downstream systems use to look up the exact response returned by a
    rule.
    """

    def __init__(self, key, **kwargs):
        self.validate_kwargs(kwargs)

        r = {"type": self.response_type}
        if self.key_name:
            self.validate_key(key)
            r[self.key_name] = key

        kwargs.update(r)
        kwargs = self.adjust_for_length(key, r, kwargs)
        super(Response, self).__init__(kwargs)

    def get_key(self):
        """
        Helper function that uses the response's key_name to look up the
        response identifier. For a rule, this is like
        response.get("error_key").
        """
        if self.key_name:
            return self.get(self.key_name)

    def validate_kwargs(self, kwargs):
        """
        Validates expected subclass attributes and constructor keyword
        arguments.
        """
        if not self.response_type:
            msg = "response_type must be set on the Response subclass."
            raise ValidationException(msg)

        if (self.key_name and self.key_name in kwargs) or "type" in kwargs:
            name = self.__class__.__name__
            msg = "%s is an invalid argument for %s" % (self.key_name, name)
            raise ValidationException(msg)

    def validate_key(self, key):
        """ Called if the key_name class attribute is not None. """
        if not key:
            name = self.__class__.__name__
            msg = "%s response missing %s" % (name, self.key_name)
            raise ValidationException(msg, self)
        elif not isinstance(key, str):
            msg = "Response contains invalid %s type" % self.key_name
            raise ValidationException(msg, type(key))

    def adjust_for_length(self, key, r, kwargs):
        """
        Converts the response to a string and compares its length to a max
        length specified in settings. If the response is too long, an error is
        logged, and an abbreviated response is returned instead.
        """
        length = len(str(kwargs))
        if length > settings.defaults["max_detail_length"]:
            self._log_length_error(key, length)
            r["max_detail_length_error"] = length
            return r
        return kwargs

    def _log_length_error(self, key, length):
        """ Helper function for logging a response length error. """
        extra = {
            "max_detail_length": settings.defaults["max_detail_length"],
            "len": length
        }
        if self.key_name:
            extra[self.key_name] = key
        msg = "Length of data in %s is too long." % self.__class__.__name__
        log.error(msg, extra=extra)


class make_response(Response):
    """
    Traditionally used by a rule to signal that its conditions have been met.
    """
    response_type = "rule"
    key_name = "error_key"


class make_fail(make_response):
    """ An alias for make_response. """
    pass


class make_pass(Response):
    """
    Can be used by a rule to explicitly indicate that a system "passed" its
    checks.
    """
    response_type = "pass"
    key_name = "pass_key"


class make_fingerprint(Response):
    response_type = "fingerprint"
    key_name = "fingerprint_key"


class make_metadata_key(Response):
    response_type = "metadata_key"
    key_name = "key"

    def __init__(self, key, value):
        super(make_metadata_key, self).__init__(key, value=value)

    def adjust_for_length(self, key, r, kwargs):
        return kwargs


class make_metadata(Response):
    """
    Allows a rule to convey addtional metadata about a system to downstream
    systems. It doesn't convey success or failure but purely information that
    may be aggregated with other make_metadata responses. As such, it has no
    response key.
    """
    response_type = "metadata"

    def __init__(self, **kwargs):
        super(make_metadata, self).__init__(None, **kwargs)


class _make_skip(Response):
    """
    Called automatically whenever a rule's dependencies aren't met. Likely to
    be deprecated or have its semantics changed. Do not call explicitly from
    rules.
    """
    response_type = "skip"

    def __init__(self, rule_fqdn, reason, details=None):
        super(_make_skip, self).__init__(None,
                                        rule_fqdn=rule_fqdn,
                                        reason=reason,
                                        details=details)



"""
SystemD service configuration
=============================

Service systemd files are in ``/usr/lib/systemd/system``, and their content
format is similar to a '.ini' file.

Parsers included in this module are:

SystemdDocker - file ``/usr/lib/systemd/system/docker.service``
---------------------------------------------------------------

SystemdOpenshiftNode - file ``/usr/lib/systemd/system/atomic-openshift-node.service``
-------------------------------------------------------------------------------------

SystemdSystemConf - file ``/etc/systemd/system.conf``
-----------------------------------------------------

"""

from insights.configtree.iniconfig import parse_doc
from insights.core import Parser, LegacyItemAccess
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.systemd_docker)
class SystemdDocker(Parser):
    """Class for docker systemd configuration.

    The content of file ``/usr/lib/systemd/system/docker.service`` is recorded
    in INI format, ``ConfigParser`` could be used to parse the content.

    Sample file content is::

        [Service]
        Type=notify
        EnvironmentFile=-/etc/sysconfig/docker
        EnvironmentFile=-/etc/sysconfig/docker-storage
        EnvironmentFile=-/etc/sysconfig/docker-network
        Environment=GOTRACEBACK=crash
        ExecStart=/bin/sh -c '/usr/bin/docker-current daemon \\
        --authorization-plugin=rhel-push-plugin \\
        --exec-opt native.cgroupdriver=systemd \\
        $OPTIONS \\
        $DOCKER_STORAGE_OPTIONS \\
        $DOCKER_NETWORK_OPTIONS \\
        $ADD_REGISTRY \\
        $BLOCK_REGISTRY \\
        $INSECURE_REGISTRY \\
        2>&1 | /usr/bin/forward-journald -tag docker'
        LimitNOFILE=1048576

    Example:
        >>> docker_content = '''
        [Service]
        Type=notify
        EnvironmentFile=-/etc/sysconfig/docker
        EnvironmentFile=-/etc/sysconfig/docker-storage
        EnvironmentFile=-/etc/sysconfig/docker-network
        Environment=GOTRACEBACK=crash
        ExecStart=/bin/sh -c '/usr/bin/docker-current daemon \\
                  --authorization-plugin=rhel-push-plugin \\
                  --exec-opt native.cgroupdriver=systemd \\
                  $OPTIONS \\
                  $DOCKER_STORAGE_OPTIONS \\
                  $DOCKER_NETWORK_OPTIONS \\
                  $ADD_REGISTRY \\
                  $BLOCK_REGISTRY \\
                  $INSECURE_REGISTRY \\
                  2>&1 | /usr/bin/forward-journald -tag docker'
        LimitNOFILE=1048576
        '''.strip()

        >>> from insights.tests import context_wrap
        >>> shared = {SystemdDocker: SystemdDocker(context_wrap(docker_content))}
        >>> docker_config_info = shared[SystemdDocker]
        >>> docker_config_info["Service"]["ExecStart"]
        "/bin/sh -c '/usr/bin/docker-current daemon --authorization-plugin=rhel-push-plugin --exec-opt native.cgroupdriver=systemd $OPTIONS $DOCKER_STORAGE_OPTIONS $DOCKER_NETWORK_OPTIONS $ADD_REGISTRY $BLOCK_REGISTRY $INSECURE_REGISTRY 2>&1 | /usr/bin/forward-journald -tag docker'"
        >>> len(docker_config_info["Service"]["EnvironmentFile"])
        3
    """

    def parse_content(self, content):
        self.data = parse_systemd_ini(content)


@parser(Specs.systemd_system_conf)
class SystemdSystemConf(LegacyItemAccess, Parser):
    """Class for systemd master configuration in the
    ``/etc/systemd/system.conf`` file.

    Systemd configuration files are recorded via INI format as well, we can
    share the same parser ``ConfigParser`` here.

    Typical output of the ``systemd_system.conf`` command is::

        [Manager]
        RuntimeWatchdogSec=0
        ShutdownWatchdogSec=10min

    Example:
        >>> systemd_content = '''
        [Manager]
        RuntimeWatchdogSec=0
        ShutdownWatchdogSec=10min
        ...
        '''.strip()

        >>> from insights.tests import context_wrap
        >>> shared = {SystemdSystemConf: SystemdSystemConf(context_wrap(systemd_content))}
        >>> systemd_config_info = shared[SystemdSystemConf]
        >>> systemd_config_info["Manager"]["RuntimeWatchdogSec"]
        "0"
    """

    def parse_content(self, content):
        self.data = parse_systemd_ini(content)

    def __contains__(self, conf):
        return conf in self.data


@parser(Specs.systemd_openshift_node)
class SystemdOpenshiftNode(Parser):
    """Class for atomic-openshift-node systemd configuration.

    The content of file ``/usr/lib/systemd/system/atomic-openshift-node.service``
    is recorded via INI format, ``ConfigParser`` could be used to parse the content.

    Typical output of the ``systemd_openshift_node`` command is::

        [Service]
        Type=notify
        RestartSec=5s
        OOMScoreAdjust=-999
        ExecStartPost=/usr/bin/sleep 10
        ExecStartPost=/usr/sbin/sysctl --system

    Example:
        >>> openshift_content = '''
        [Service]
        Type=notify
        RestartSec=5s
        OOMScoreAdjust=-999
        ExecStartPost=/usr/bin/sleep 10
        ExecStartPost=/usr/sbin/sysctl --system
        ...
        '''.strip()

        >>> from insights.tests import context_wrap
        >>> shared = {SystemdOpenshiftNode: SystemdOpenshiftNode(context_wrap(openshift_content))}
        >>> openshift_config_info = shared[SystemdOpenshiftNode]
        >>> openshift_config_info["Service"]["RestartSec"]
        "5s"
        >>> len(openshift_config_info["Service"]["ExecStartPost"])
        2
    """

    def parse_content(self, content):
        self.data = parse_systemd_ini(content)


class MultiOrderedDict(dict):
    """Class for condition that duplicate keys exist"""

    def __setitem__(self, key, value):
        if isinstance(value, list) and key in self:
            self[key].extend(value)
        else:
            super(MultiOrderedDict, self).__setitem__(key, value)


def parse_systemd_ini(content):
    """Function to parse config format file, the result format is dictionary"""

    doc = parse_doc(content)

    dict_all = {}
    for section in doc:
        section_dict = {}
        option_names = set(o.name for o in section)
        for name in option_names:
            options = [str(o.value) for o in section[name]]
            section_dict[name] = options[0] if len(options) == 1 else options
        dict_all[section.name] = section_dict

    return dict_all



#!/usr/bin/env python
from insights import run
from insights.parsers.installed_rpms import InstalledRpms
from insights.combiners.hostname import hostname
from insights.core.plugins import fact, rule


@fact(InstalledRpms)
def bash_version(rpms):
    rpm = rpms.get_max("bash")
    return {"name": rpm.name, "version": rpm.nvr}


@fact(hostname)
def get_hostname(hn):
    return {"hostname": hn.fqdn}


@rule(bash_version, get_hostname, cluster=True)
def bash_rule(bash, hn):
    pass


if __name__ == "__main__":
    run(bash_version, print_summary=True)



import pytest
from insights.parsers.grubby import GrubbyDefaultIndex
from insights.tests import context_wrap
from insights.parsers import SkipException

DEFAULT_INDEX_1 = '0'
DEFAULT_INDEX_2 = '1'
DEFAULT_INDEX_3 = ''
DEFAULT_INDEX_4 = '-2'


def test_grub2_default_index_1():
    res = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_1))
    assert res.default_index == 0

    res = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_2))
    assert res.default_index == 1


def test_grub2_default_index_2():
    with pytest.raises(SkipException) as excinfo:
        GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_3))
        assert 'Invalid default index value.' in str(excinfo.value)

    with pytest.raises(SkipException) as excinfo:
        GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_4))
        assert 'Invalid default index value.' in str(excinfo.value)



from insights.tests import context_wrap
from insights.parsers import md5check

PRELINK_SAMPLE = """
039a62d3e9786f5a63e84def09cb3b27  /usr/lib64/libfreeblpriv3.so
""".strip()

MD5_SAMPLE = """
4777325b4a49cfae41a3991887920b68  /usr/lib64/libfreebl3.so
""".strip()

BAD_INPUT_SAMPLE = """
""".strip()

FILES_NOT_FOUND_SAMPLE = """
d41d8cd98f00b204e9800998ecf8427e  /dev/null
""".strip()


def test_md5check_prelink():
    md5info = md5check.MD5CheckSum(context_wrap(PRELINK_SAMPLE))
    assert md5info.data['md5sum'] == '039a62d3e9786f5a63e84def09cb3b27'
    assert md5info.data['filename'] == '/usr/lib64/libfreeblpriv3.so'
    assert md5info.md5sum == '039a62d3e9786f5a63e84def09cb3b27'
    assert md5info.filename == '/usr/lib64/libfreeblpriv3.so'


def test_md5check_md5():
    md5info = md5check.MD5CheckSum(context_wrap(MD5_SAMPLE))
    assert md5info.data['md5sum'] == '4777325b4a49cfae41a3991887920b68'
    assert md5info.data['filename'] == '/usr/lib64/libfreebl3.so'
    assert md5info.md5sum == '4777325b4a49cfae41a3991887920b68'
    assert md5info.filename == '/usr/lib64/libfreebl3.so'


def test_md5check_bad_input():
    md5info = md5check.MD5CheckSum(context_wrap(BAD_INPUT_SAMPLE))
    assert md5info.data == {}
    assert md5info.md5sum is None
    assert md5info.filename is None


def test_md5check_files_not_found():
    md5info = md5check.MD5CheckSum(context_wrap(FILES_NOT_FOUND_SAMPLE))
    assert md5info.data == {}
    assert md5info.md5sum is None
    assert md5info.filename is None



"""
This module contains logic for parsing ls output. It attempts to handle
output when selinux is enabled or disabled and also skip "bad" lines.
"""
import logging

log = logging.getLogger(__name__)
PERMBITS = set("-+.dlbcpsrwxtT")


def parse_path(path):
    """
    Convert possible symbolic link into a source -> target pair.

    Args:
        path (str): The path portion of an ls output line.

    Returns:
        A (path, link) tuple where path is always populated and link is a non
        empty string if the original path is a symoblic link.
    """
    path, _, link = path.partition(" -> ")
    return path, link


def parse_non_selinux(parts):
    """
    Parse part of an ls output line that isn't selinux.

    Args:
        parts (list): A four element list of strings representing the initial
            parts of an ls line after the permission bits. The parts are link
            count, owner, group, and everything else.

    Returns:
        A dict containing links, owner, group, date, and name. If the line
        represented a device, major and minor numbers are included.  Otherwise,
        size is included. If the raw name was a symbolic link, link is
        included.
    """
    links, owner, group, last = parts
    result = {
        "links": int(links),
        "owner": owner,
        "group": group,
    }

    # device numbers only go to 256.
    # If a comma is in the first four characters, the next two elements are
    # major and minor device numbers. Otherwise, the next element is the size.
    if "," in last[:4]:
        major, minor, rest = last.split(None, 2)
        result["major"] = int(major.rstrip(","))
        result["minor"] = int(minor)
    else:
        size, rest = last.split(None, 1)
        result["size"] = int(size)

    # The date part is always 12 characters regardless of content.
    result["date"] = rest[:12]

    # Jump over the date and the following space to get the path part.
    path, link = parse_path(rest[13:])
    result["name"] = path
    if link:
        result["link"] = link

    return result


def parse_selinux(parts):
    """
    Parse part of an ls output line that is selinux.

    Args:
        parts (list): A four element list of strings representing the initial
            parts of an ls line after the permission bits. The parts are owner
            group, selinux info, and the path.

    Returns:
        A dict containing owner, group, se_user, se_role, se_type, se_mls, and
        name. If the raw name was a symbolic link, link is always included.

    """

    owner, group = parts[:2]
    selinux = parts[2].split(":")
    lsel = len(selinux)
    path, link = parse_path(parts[-1])
    result = {
        "owner": owner,
        "group": group,
        "se_user": selinux[0],
        "se_role": selinux[1] if lsel > 1 else None,
        "se_type": selinux[2] if lsel > 2 else None,
        "se_mls": selinux[3] if lsel > 3 else None,
        "name": path
    }
    if link:
        result["link"] = link
    return result


def parse(lines, root):
    """
    Parses a list of lines from ls into dictionaries representing their
    components.

    Args:
        lines (list): A list of lines generated by ls.
        root (str): The directory name to be used for ls output stanzas that
            don't have a name.

    Returns:
        A dictionary representing the ls output. It's keyed by the path
        containing each ls stanza.
    """
    lines = iter(lines)
    results = {}
    # we process until StopIteration exception is thrown by the line iterator.
    while True:
        try:
            # get a line and skip blanks
            line = next(lines)
            while not(line):
                line = next(lines)
            try:
                # an ls stanza begins with an optional directory name followed
                # by a total number of entries in the directory and then the
                # list of entries. Stanzas are separated by blank lines.
                name = None
                total = None
                if line.endswith(":"):
                    name = line[:-1]
                    line = next(lines).strip()
                if line.startswith("total"):
                    total = int(line.split()[1])
                    line = next(lines).strip()

                # If we didn't get a name or a total, it's not a valid stanza
                # start, so just start over.
                if not name and not total:
                    continue

                # handle unnamed ls stanzas
                name = name or root

                # keep track of directories, files, special entries like block
                # and character devices, and a master list of all entries
                # regardless of kind.
                dirs = []
                ents = {}
                files = []
                specials = []

                # make sure we have a good line and its first character is one
                # of the valid permission bits.
                while line and line[0] in PERMBITS:
                    # break the line into initial parts
                    parts = line.split(None, 4)

                    # Get the permissions and validate they're all valid bits.
                    # Otherwise, our single character check in the loop was
                    # fooled, and we should skip the line as invalid.
                    perms = parts[0]
                    if len(perms) < 10:
                        line = next(lines).strip()
                        continue

                    # First character of the perms is the type of entry:
                    # directory, regular file, etc. The rest are the actual
                    # permission bits.
                    typ = perms[0]
                    entry = {
                        "type": typ,
                        "perms": perms[1:]
                    }

                    # If the first character of the second part of the line is
                    # a digit, then it's a link count and indicates we're not
                    # dealing with selinux. Otherwise, it's an owner, and we
                    # are dealing with selinux.
                    if parts[1][0].isdigit():
                        rest = parse_non_selinux(parts[1:])
                    else:
                        rest = parse_selinux(parts[1:])

                    # Update our entry and put it into the correct buckets
                    # based on its type.
                    entry.update(rest)
                    entry["raw_entry"] = line
                    entry["dir"] = name
                    nm = entry["name"]
                    ents[nm] = entry
                    if typ not in "bcd":
                        files.append(nm)
                    elif typ == "d":
                        dirs.append(nm)
                    elif typ in "bc":
                        specials.append(nm)
                    try:
                        line = next(lines).strip()
                    except StopIteration:
                        break

                # We're either at the end of input or a blank line, so we wrap
                # up this stanza and save it.
                if total is None:
                    total = len(ents)

                result = {
                    "name": name,
                    "total": total,
                    "entries": ents,
                    "files": files,
                    "dirs": dirs,
                    "specials": specials
                }
                results[result["name"]] = result

            # If we're at the end of input, blow all the way out.
            except StopIteration:
                raise

            # Something exploded. Just log it and try to move on.
            except:
                line = next(lines)
                log.info("Failed to parse: %s" % line)
        except StopIteration:
            break
    return results



"""
MD5CheckSum - md5 checksums of specified binary or library files
================================================================

Module for processing output of the ``prelink -y --md5`` or ``md5sum`` command.

The name and md5 checksums of specified files are stored as lists. Each
entry will be processed by the shared parser.

This MD5CheckSum class provide the following properties:

* ``filename``: the file name which derived from the output.
* ``md5sum``: the checksums for the file.
"""
from .. import parser, CommandParser
from insights.specs import Specs


@parser(Specs.prelink_orig_md5)
class MD5CheckSum(CommandParser):
    """
    Class to parse ``prelink -y --md5`` or ``md5sum`` command information.

    The output of these two commands contains two fields, the first is the
    md5 checksum and the second is the file name.

    Especially, we add /dev/null as one of the specified file to be checked,
    as ``md5sum`` command can read from stdin without feeding a file.

    Attributes::

        filename (str): File name.
        md5sum (str): MD5 checksums of that file.

    Sample input for ``prelink -y --md5`` or ``md5sum`` command::

        d1e6613cfb62d3f111db7bdda39ac821  /usr/lib64/libsoftokn3.so

    Examples::

        >>> collection = shared[MD5CheckSum]
        >>> len(collection) # All MD5 checksums in a list
        1
        >>> record = collection[0]
        >>> record.filename
        /usr/lib64/libsoftokn3.so
        >>> record.md5sum
        d1e6613cfb62d3f111db7bdda39ac821

    """
    @property
    def filename(self):
        """ str: Return the file name for the current file."""
        return self.data.get('filename')

    @property
    def md5sum(self):
        """ str: Return the checksum for the current file."""
        return self.data.get('md5sum')

    def parse_content(self, content):
        self.data = {}
        fields = content[0].strip().split() if content else None
        # skip the '/dev/null'
        if fields and len(fields) == 2 and fields[1] != "/dev/null":
            self.data["md5sum"] = fields[0]
            self.data["filename"] = fields[1]



from __future__ import print_function
import pkgutil
import os
import yaml
from .core import Scannable, LogFileOutput, Parser, IniConfigFile  # noqa: F401
from .core import FileListing, LegacyItemAccess, SysconfigOptions  # noqa: F401
from .core import YAMLParser, JSONParser, XMLParser, CommandParser  # noqa: F401
from .core import AttributeDict  # noqa: F401
from .core import Syslog  # noqa: F401
from .core.archives import COMPRESSION_TYPES, extract  # noqa: F401
from .core import dr  # noqa: F401
from .core.context import ClusterArchiveContext, HostContext, HostArchiveContext  # noqa: F401
from .core.dr import SkipComponent  # noqa: F401
from .core.hydration import create_context
from .core.plugins import combiner, fact, metadata, parser, rule  # noqa: F401
from .core.plugins import datasource, condition, incident  # noqa: F401
from .core.plugins import make_response, make_metadata, make_fingerprint  # noqa: F401
from .core.plugins import make_pass, make_fail  # noqa: F401
from .core.filters import add_filter, apply_filters, get_filters  # noqa: F401
from .formats import get_formatter
from .parsers import get_active_lines  # noqa: F401
from .util import defaults  # noqa: F401


package_info = dict((k, None) for k in ["RELEASE", "COMMIT", "VERSION", "NAME"])


for name in package_info:
    package_info[name] = pkgutil.get_data(__name__, name).strip().decode("utf-8")


def get_nvr():
    return "{0}-{1}-{2}".format(package_info["NAME"],
                                package_info["VERSION"],
                                package_info["RELEASE"])


RULES_STATUS = {}
"""
Mapping of dictionaries containing nvr and commitid for each rule repo included
in this instance

{"rule_repo_1": {"version": nvr(), "commit": sha1}}
"""


def add_status(name, nvr, commit):
    """
    Rule repositories should call this method in their package __init__ to
    register their version information.
    """
    RULES_STATUS[name] = {"version": nvr, "commit": commit}


def process_dir(broker, root, graph, context, use_pandas=False):
    ctx = create_context(root, context)

    if isinstance(ctx, ClusterArchiveContext):
        from .core.cluster import process_cluster
        archives = [f for f in ctx.all_files if f.endswith(COMPRESSION_TYPES)]
        return process_cluster(archives, use_pandas=use_pandas, broker=broker)

    broker[ctx.__class__] = ctx
    broker = dr.run(graph, broker=broker)
    return broker


def _run(broker, graph=None, root=None, context=None, use_pandas=False):
    """
    run is a general interface that is meant for stand alone scripts to use
    when executing insights components.

    Args:
        root (str): None will causes a host collection in which command and
            file specs are run. A directory or archive path will cause
            collection from the directory or archive, and only file type specs
            or those that depend on `insights.core.context.HostArchiveContext`
            will execute.
        component (function or class): The component to execute. Will only execute
            the component and its dependency graph. If None, all components with
            met dependencies will execute.

    Returns:
        broker: object containing the result of the evaluation.
    """

    if not root:
        context = context or HostContext
        broker[context] = context()
        return dr.run(graph, broker=broker)

    if os.path.isdir(root):
        return process_dir(broker, root, graph, context, use_pandas)
    else:
        with extract(root) as ex:
            return process_dir(broker, ex.tmp_dir, graph, context, use_pandas)


def apply_configs(configs):
    """
    Configures components. They can be enabled or disabled, have timeouts set
    if applicable, and have metadata customized. Valid keys are name, enabled,
    metadata, and timeout.

    Args:
        configs (list): a list of dictionaries with the following keys:
            name, enabled, metadata, and timeout. All keys are optional except
            name.

            name is the prefix or exact name of any loaded component. Any
            component starting with name will have the associated configuration
            applied.

            enabled is whether the matching components will execute even if
            their dependencies are met. Defaults to True.

            timeout sets the class level timeout attribute of any component so
            long as the attribute already exists.

            metadata is any dictionary that you want to attach to the
            component. The dictionary can be retrieved by the component at
            runtime.
    """
    delegate_keys = sorted(dr.DELEGATES, key=dr.get_name)
    for comp_cfg in configs:
        name = comp_cfg["name"]
        for c in delegate_keys:
            delegate = dr.DELEGATES[c]
            cname = dr.get_name(c)
            if cname.startswith(name):
                dr.ENABLED[c] = comp_cfg.get("enabled", True)
                delegate.metadata.update(comp_cfg.get("metadata", {}))
                for k, v in delegate.metadata.items():
                    if hasattr(c, k):
                        setattr(c, k, v)
                if hasattr(c, "timeout"):
                    c.timeout = comp_cfg.get("timeout")
            if cname == name:
                break


def _load_context(path):
    if path is None:
        return

    if "." not in path:
        path = ".".join(["insights.core.context", path])
    return dr.get_component(path)


def run(component=None, root=None, print_summary=False,
        context=None, use_pandas=False,
        print_component=None):

    from .core import dr
    dr.load_components("insights.specs.default")
    dr.load_components("insights.specs.insights_archive")
    dr.load_components("insights.specs.sos_archive")
    dr.load_components("insights.specs.jdr_archive")

    args = None
    formatter = None
    if print_summary:
        import argparse
        import logging
        p = argparse.ArgumentParser(add_help=False)
        p.add_argument("archive", nargs="?", help="Archive or directory to analyze.")
        p.add_argument("-p", "--plugins", default="", help="Comma-separated list without spaces of package(s) or module(s) containing plugins.")
        p.add_argument("-c", "--config", help="Configure components.")
        p.add_argument("-v", "--verbose", help="Verbose output.", action="store_true")
        p.add_argument("-f", "--format", help="Output format.", default="insights.formats.text")
        p.add_argument("-D", "--debug", help="Verbose debug output.", action="store_true")
        p.add_argument("--context", help="Execution Context. Defaults to HostContext if an archive isn't passed.")
        p.add_argument("--pandas", action="store_true", help="Use pandas dataframes with cluster rules.")

        class Args(object):
            pass

        args = Args()
        p.parse_known_args(namespace=args)
        p = argparse.ArgumentParser(parents=[p])
        args.format = "insights.formats._json" if args.format == "json" else args.format
        args.format = "insights.formats._yaml" if args.format == "yaml" else args.format
        fmt = args.format if "." in args.format else "insights.formats." + args.format
        Formatter = dr.get_component(fmt)
        if not Formatter:
            dr.load_components(fmt, continue_on_error=False)
            Formatter = get_formatter(fmt)
        Formatter.configure(p)
        p.parse_args(namespace=args)
        formatter = Formatter(args)

        logging.basicConfig(level=logging.DEBUG if args.debug else logging.INFO if args.verbose else logging.ERROR)
        context = _load_context(args.context) or context
        use_pandas = args.pandas or use_pandas

        root = args.archive or root
        if root:
            root = os.path.realpath(root)

        plugins = []
        if args.plugins:
            for path in args.plugins.split(","):
                path = path.strip()
                if path.endswith(".py"):
                    path, _ = os.path.splitext(path)
                path = path.rstrip("/").replace("/", ".")
                plugins.append(path)

        for p in plugins:
            dr.load_components(p)

        if args.config:
            with open(args.config) as f:
                apply_configs(yaml.load(f))

        if component is None:
            component = []
            plugins = tuple(plugins)
            for c in dr.DELEGATES:
                if c.__module__.startswith(plugins):
                    component.append(c)

    if component:
        if not isinstance(component, (list, set)):
            component = [component]
        graph = {}
        for c in component:
            graph.update(dr.get_dependency_graph(c))
    else:
        graph = dr.COMPONENTS[dr.GROUPS.single]

    broker = dr.Broker()

    if formatter:
        formatter.preprocess(broker)
        broker = _run(broker, graph, root, context=context, use_pandas=use_pandas)
        formatter.postprocess(broker)
    elif print_component:
        broker = _run(broker, graph, root, context=context, use_pandas=use_pandas)
        broker.print_component(print_component)
    else:
        broker = _run(broker, graph, root, context=context, use_pandas=use_pandas)

    return broker


def main():
    run(print_summary=True)


if __name__ == "__main__":
    main()



import json
import os
import logging
import tempfile
import shlex
import shutil
import sys
from subprocess import Popen, PIPE

from .. import package_info
from . import client
from .constants import InsightsConstants as constants
from .config import InsightsConfig

logger = logging.getLogger(__name__)
net_logger = logging.getLogger("network")


class InsightsClient(object):

    def __init__(self, config, setup_logging=True, **kwargs):
        """
        The Insights client interface
        """
        #  small hack to maintain compatability with RPM wrapper
        if isinstance(config, InsightsConfig):
            self.config = config
        else:
            try:
                self.config = InsightsConfig(_print_errors=True).load_all()
            except ValueError as e:
                sys.stderr.write('ERROR: ' + str(e) + '\n')
                sys.exit(constants.sig_kill_bad)
        # end hack. in the future, just set self.config=config

        # set up logging
        if setup_logging:
            self.set_up_logging()

        # setup insights connection placeholder
        # used for requests
        self.session = None
        self.connection = None

    def _net(func):
        def _init_connection(self, *args, **kwargs):
            # setup a request session
            if not self.config.offline and not self.session:
                self.connection = client.get_connection(self.config)
                self.session = self.connection.session
            return func(self, *args, **kwargs)
        return _init_connection

    def get_conf(self):
        # need this getter to maintain compatability with RPM wrapper
        return self.config

    def set_up_logging(self):
        return client.set_up_logging(self.config)

    def version(self):
        return "%s-%s" % (package_info["VERSION"], package_info["RELEASE"])

    @_net
    def test_connection(self):
        """
            returns (int): 0 if success 1 if failure
        """
        return self.connection.test_connection()

    @_net
    def branch_info(self):
        """
            returns (dict): {'remote_leaf': -1, 'remote_branch': -1}
        """
        return client.get_branch_info(self.config, self.connection)

    def fetch(self, force=False):
        """
            returns (dict): {'core': path to new egg, None if no update,
                             'gpg_sig': path to new sig, None if no update}
        """
        tmpdir = tempfile.mkdtemp()
        fetch_results = {
            'core': os.path.join(tmpdir, 'insights-core.egg'),
            'gpg_sig': os.path.join(tmpdir, 'insights-core.egg.asc')
        }

        logger.debug("Beginning core fetch.")

        # run fetch for egg
        updated = self._fetch(self.config.egg_path,
                              constants.core_etag_file,
                              fetch_results['core'],
                              force)

        # if new core was fetched, get new core sig
        if updated:
            logger.debug("New core was fetched.")
            logger.debug("Beginning fetch for core gpg signature.")
            self._fetch(self.config.egg_gpg_path,
                        constants.core_gpg_sig_etag_file,
                        fetch_results['gpg_sig'],
                        force)

            return fetch_results

    @_net
    def _fetch(self, path, etag_file, target_path, force):
        """
            returns (str): path to new egg. None if no update.
        """
        url = self.connection.base_url + path
        # Searched for cached etag information
        current_etag = None
        if os.path.isfile(etag_file):
            with open(etag_file, 'r') as fp:
                current_etag = fp.read().strip()
                logger.debug('Found etag %s', current_etag)

        # Setup the new request for core retrieval
        logger.debug('Making request to %s for new core', url)

        # If the etag was found and we are not force fetching
        # Then add it to the request
        net_logger.info("GET %s", url)
        if current_etag and not force:
            logger.debug('Requesting new file with etag %s', current_etag)
            etag_headers = {'If-None-Match': current_etag}
            response = self.session.get(url, headers=etag_headers)
        else:
            logger.debug('Found no etag or forcing fetch')
            response = self.session.get(url)

        # Debug information
        logger.debug('Status code: %d', response.status_code)
        for header, value in response.headers.items():
            logger.debug('%s: %s', header, value)

        # Debug the ETag
        logger.debug('ETag: %s', response.request.headers.get('If-None-Match'))

        # If data was received, write the new egg and etag
        if response.status_code == 200 and len(response.content) > 0:

            # Write the new core
            with open(target_path, 'wb') as handle:
                logger.debug('Data received, writing core to %s', target_path)
                handle.write(response.content)

            # Write the new etag
            with open(etag_file, 'w') as handle:
                logger.debug('Cacheing etag to %s', etag_file)
                handle.write(response.headers['etag'])

            return True

        # Received a 304 not modified
        # Return nothing
        elif response.status_code == 304:
            logger.debug('No data received')
            logger.debug('Tags match, not updating core')

        # Something unexpected received
        else:
            logger.debug('Received Code %s', response.status_code)
            logger.debug('Not writing new core, or updating etag')
            logger.debug('Please check config, error reaching %s', url)

    def update(self):
        # dont update if running in offline mode
        if self.config.offline:
            logger.debug("Not updating Core. Running in offline mode.")
            return True

        if self.config.auto_update:
            # fetch the new eggs and gpg
            egg_paths = self.fetch()

            # if the gpg checks out install it
            if (egg_paths and self.verify(egg_paths['core'])['gpg']):
                return self.install(egg_paths['core'], egg_paths['gpg_sig'])
            else:
                return False
        else:
            logger.debug("Egg update disabled")

    def verify(self, egg_path, gpg_key=constants.pub_gpg_path):
        """
            Verifies the GPG signature of the egg.  The signature is assumed to
            be in the same directory as the egg and named the same as the egg
            except with an additional ".asc" extension.

            returns (dict): {'gpg': if the egg checks out,
                             'stderr': error message if present,
                             'stdout': stdout,
                             'rc': return code}
        """
        # check if the provided files (egg and gpg) actually exist
        if egg_path and not os.path.isfile(egg_path):
            the_message = "Provided egg path %s does not exist, cannot verify." % (egg_path)
            logger.debug(the_message)
            return {'gpg': False,
                    'stderr': the_message,
                    'stdout': the_message,
                    'rc': 1,
                    'message': the_message}
        if self.config.gpg and gpg_key and not os.path.isfile(gpg_key):
            the_message = ("Running in GPG mode but cannot find "
                            "file %s to verify against." % (gpg_key))
            logger.debug(the_message)
            return {'gpg': False,
                    'stderr': the_message,
                    'stdout': the_message,
                    'rc': 1,
                    'message': the_message}

        # if we are running in no_gpg or not gpg mode then return true
        if not self.config.gpg:
            return {'gpg': True,
                    'stderr': None,
                    'stdout': None,
                    'rc': 0}

        # if a valid egg path and gpg were received do the verification
        if egg_path and gpg_key:
            cmd_template = '/usr/bin/gpg --verify --keyring %s %s %s'
            cmd = cmd_template % (gpg_key, egg_path + '.asc', egg_path)
            logger.debug(cmd)
            process = Popen(shlex.split(cmd), stdout=PIPE, stderr=PIPE)
            stdout, stderr = process.communicate()
            rc = process.returncode
            logger.debug("GPG return code: %s" % rc)
            return {'gpg': True if rc == 0 else False,
                    'stderr': stderr,
                    'stdout': stdout,
                    'rc': rc}
        else:
            return {'gpg': False,
                    'stderr': 'Must specify a valid core and gpg key.',
                    'stdout': 'Must specify a valid core and gpg key.',
                    'rc': 1}

    def install(self, new_egg, new_egg_gpg_sig):
        """
        returns (dict): {'success': True if the core installation successfull else False}
        raises OSError if cannot create /var/lib/insights
        raises IOError if cannot copy /tmp/insights-core.egg to /var/lib/insights/newest.egg
        """

        # make sure a valid egg was provided
        if not new_egg:
            the_message = 'Must provide a valid Core installation path.'
            logger.debug(the_message)
            return {'success': False, 'message': the_message}

        # if running in gpg mode, check for valid sig
        if self.config.gpg and new_egg_gpg_sig is None:
            the_message = "Must provide a valid Core GPG installation path."
            logger.debug(the_message)
            return {'success': False, 'message': the_message}

        # debug
        logger.debug("Installing the new Core %s", new_egg)
        if self.config.gpg:
            logger.debug("Installing the new Core GPG Sig %s", new_egg_gpg_sig)

        # Make sure /var/lib/insights exists
        try:
            if not os.path.isdir(constants.insights_core_lib_dir):
                logger.debug("Creating directory %s for the Core." %
                             (constants.insights_core_lib_dir))
                os.mkdir(constants.insights_core_lib_dir)
        except OSError:
            logger.info("There was an error creating %s for core installation." % (
                constants.insights_core_lib_dir))
            raise

        # Copy the NEW (/tmp/insights-core.egg) egg to /var/lib/insights/newest.egg
        # Additionally, copy NEW (/tmp/insights-core.egg.asc) to /var/lib/insights/newest.egg.asc
        try:
            logger.debug("Copying %s to %s." % (new_egg, constants.insights_core_newest))
            shutil.copyfile(new_egg, constants.insights_core_newest)
            shutil.copyfile(new_egg_gpg_sig, constants.insights_core_gpg_sig_newest)
        except IOError:
            logger.info("There was an error copying the new core from %s to %s." % (
                new_egg, constants.insights_core_newest))
            raise

        logger.debug("The new Insights Core was installed successfully.")
        return {'success': True}

    @_net
    def update_rules(self):
        """
            returns (dict): new client rules
        """
        if self.config.offline or not self.config.auto_update:
            logger.debug("Bypassing rule update due to config "
                "running in offline mode or auto updating turned off.")
        else:
            return client.update_rules(self.config, self.connection)

    @_net
    def collect(self):
        # return collection results
        tar_file = client.collect(self.config, self.connection)

        # it is important to note that --to-stdout is utilized via the wrapper RPM
        # this file is received and then we invoke shutil.copyfileobj
        return tar_file

    @_net
    def register(self):
        """
            returns (bool | None):
                True - machine is registered
                False - machine is unregistered
                None - could not reach the API
        """
        return client.handle_registration(self.config, self.connection)

    @_net
    def unregister(self):
        """
            returns (bool): True success, False failure
        """
        return client.handle_unregistration(self.config, self.connection)

    @_net
    def upload(self, path):
        """
            returns (int): upload status code
        """
        # do the upload
        upload_results = client.upload(self.config, self.connection, path)

        # return api response
        return upload_results

    def rotate_eggs(self):
        """
            moves newest.egg to last_stable.egg
            this is used by the upload() function upon 2XX return
            returns (bool): if eggs rotated successfully
            raises (IOError): if it cant copy the egg from newest to last_stable
        """
        # make sure the library directory exists
        if os.path.isdir(constants.insights_core_lib_dir):
            # make sure the newest.egg exists
            if os.path.isfile(constants.insights_core_newest):
                # try copying newest to latest_stable
                try:
                    # copy the core
                    shutil.move(constants.insights_core_newest,
                             constants.insights_core_last_stable)
                    # copy the core sig
                    shutil.move(constants.insights_core_gpg_sig_newest,
                             constants.insights_core_last_stable_gpg_sig)
                except IOError:
                    message = ("There was a problem copying %s to %s." %
                                (constants.insights_core_newest,
                                constants.insights_core_last_stable))
                    logger.debug(message)
                    raise IOError(message)
                return True
            else:
                message = ("Cannot copy %s to %s because %s does not exist." %
                            (constants.insights_core_newest,
                            constants.insights_core_last_stable,
                            constants.insights_core_newest))
                logger.debug(message)
                return False
        else:
            logger.debug("Cannot copy %s to %s because the %s directory does not exist." %
                (constants.insights_core_newest,
                    constants.insights_core_last_stable,
                    constants.insights_core_lib_dir))
            logger.debug("Try installing the Core first.")
            return False

    def get_last_upload_results(self):
        """
            returns (json): returns last upload json results or False
        """
        if os.path.isfile(constants.last_upload_results_file):
            logger.debug('Last upload file %s found, reading results.', constants.last_upload_results_file)
            with open(constants.last_upload_results_file, 'r') as handler:
                return handler.read()
        else:
            logger.debug('Last upload file %s not found, cannot read results', constants.last_upload_results_file)
            return False

    def delete_archive(self, path, delete_parent_dir=False):
        """
            returns (bool): successful archive deletion
        """
        return client.delete_archive(path, delete_parent_dir)

    @_net
    def get_registration_status(self):
        """
            returns (json):
                {'messages': [dotfile message, api message],
                 'status': (bool) registered = true; unregistered = false
                 'unreg_date': Date the machine was unregistered | None,
                 'unreachable': API could not be reached}
        """
        return client.get_registration_status(self.config, self.connection)

    @_net
    def set_display_name(self, display_name):
        '''
            returns True on success, False on failure
        '''
        return self.connection.set_display_name(display_name)


def format_config(config):
    # Log config except the password
    # and proxy as it might have a pw as well
    config_copy = config.copy()
    try:
        del config_copy["password"]
        del config_copy["proxy"]
    finally:
        return json.dumps(config_copy, indent=4)



from insights import make_response
from insights.specs import Specs
from insights.core.cluster import ClusterMeta
from insights.core.plugins import fact, rule
from insights.util.fs import sha256

CONTENT = {
    "DISTINCT_NTP_CONFS": "{{confs}} distinct NTP configurations of {{nodes}} members."
}


@fact(Specs.ntp_conf)
def ntp_sha256(ntp):
    return {"sha": sha256(ntp.path)}


@rule(ntp_sha256, ClusterMeta, cluster=True)
def report(shas, meta):
    num_members = meta.num_members
    uniq = shas.sha.unique()
    if len(shas) != num_members or len(uniq) != 1:
        return make_response("DISTINCT_NTP_CONFS", confs=len(uniq), nodes=num_members)


if __name__ == "__main__":
    from insights import run
    run(report, print_summary=True)



from insights.parsers.docker_storage_setup import DockerStorageSetup
from insights.tests import context_wrap

DOCKER_STORAGE_SETUP1 = """
# Edit this file to override any configuration options specified in
# /usr/lib/docker-storage-setup/docker-storage-setup.
#
# For more details refer to "man docker-storage-setup"
VG=vgtest
AUTO_EXTEND_POOL=yes
##name = mydomain
POOL_AUTOEXTEND_THRESHOLD=60
POOL_AUTOEXTEND_PERCENT=20
""".strip()

DOCKER_STORAGE_SETUP2 = """
#comment
# comment
# comment = comment
# comment = comment = comment
#comment=comment
#comment=comment=comment
option_a=value_a
option_b = value_b
option_c= value_c
option_d =value_d
broken_option_e = value_e = value_2_e
broken_option_f=value_f=value_2_f
broken_option_g
option_h = value_h # some comment
option_i = value_i # this must be accessible, even after all these errors
""".strip()

DOCKER_STORAGE_SETUP_PATH = "etc/sysconfig/docker-storage-setup"


def test_constructor():
    context = context_wrap(DOCKER_STORAGE_SETUP1, DOCKER_STORAGE_SETUP_PATH)
    result = DockerStorageSetup(context)

    assert 'POOL_AUTOEXTEND_THRESHOLD' in result.data
    assert "20" == result.data["POOL_AUTOEXTEND_PERCENT"]
    assert "name" not in result.data
    assert "##name" not in result.data
    assert "vgtest" == result["VG"]

    context = context_wrap(DOCKER_STORAGE_SETUP2, DOCKER_STORAGE_SETUP_PATH)
    result = DockerStorageSetup(context)

    assert "comment" not in result
    assert "broken_option_g" not in result
    # Options with spaces around the '=' are not allowed in /etc/sysconfig
    assert "option_i" not in result


def build_active_settings_expected():
    active_settings = {}
    for line in DOCKER_STORAGE_SETUP1.split("\n"):
        if not line.strip().startswith("#"):
            if line.strip():
                try:
                    key, value = line.split("=", 1)
                    key, value = key.strip(), value.strip()
                except:
                    pass
                else:
                    active_settings[key] = value
    return active_settings


def test_active_settings():
    # Paul Wayper 2017-02-23: I'm really not sure what the point of trying to
    # rewrite a parser for these files is.  We should just be testing the
    # data we expect in the file.
    context = context_wrap(DOCKER_STORAGE_SETUP1, DOCKER_STORAGE_SETUP_PATH)
    result = DockerStorageSetup(context)
    active_settings = build_active_settings_expected()
    assert active_settings == result.data



import json

from insights.formats import EvaluatorFormatterAdapter, EvaluatorFormatter


class JsonFormat(EvaluatorFormatter):
    def dump(self, data):
        return json.dumps(data)


class JsonFormatterAdapter(EvaluatorFormatterAdapter):
    Impl = JsonFormat



import yaml

from insights.formats import EvaluatorFormatterAdapter, EvaluatorFormatter


class YamlFormat(EvaluatorFormatter):
    def dump(self, data):
        return yaml.dump(data)


class YamlFormatterAdapter(EvaluatorFormatterAdapter):
    Impl = YamlFormat
